// 255-Minute Binary Countdown with Alarm
// by Ido Gendel, 2012
// Share and Enjoy!

// SN74LS164N Interface
// Pin scheme for ATTiny85
#define SCLK_PIN  4
#define DATA_PIN  2
// LED 1 & 2 Exchangeable Reference 
#define XREF_PIN 1
// Speaker/Buzzer (PWM)
#define SPKR_PIN 0
// Buttons Input
#define BTN_PIN 3

// Adjusting for internal oscillator inaccuracy
const long int SECOND = 1034;
const long int MINUTE = 60 * SECOND;
const long int ANNOY_TIME = 30 * SECOND;

struct tButton {
  boolean state;
  boolean clicked;
  long int msCount;  
} button[2];

byte t = 0;
long int clockCount;

//-------------------------------------------------------

void initButtons() {
  
   for (byte i = 0; i < 2; i++) {
     button[i].state = 0;
     button[i].clicked = false;
  }
  
}

//-------------------------------------------------------

void displayByte(byte b) {
  
   if (b > 3) {
     b ^= 3; 
     digitalWrite(XREF_PIN, HIGH);
   } else digitalWrite(XREF_PIN, LOW); 
  shiftOut(DATA_PIN, SCLK_PIN, MSBFIRST, b);

}

//-------------------------------------------------------

void setup() {
  
   pinMode(SCLK_PIN, OUTPUT); 
   digitalWrite(SCLK_PIN, LOW);
   pinMode(DATA_PIN, OUTPUT);
   pinMode(XREF_PIN, OUTPUT); 
   pinMode(SPKR_PIN, OUTPUT); 
   digitalWrite(SPKR_PIN, LOW);
   initButtons();
   setTime(0);
  
}

//-------------------------------------------------------

void setTime(const byte newTime) {

  t = newTime;
  clockCount = millis();
  displayByte(t);  
  
}

//-------------------------------------------------------

boolean buttonClicked(const byte i) {
  
  boolean isClicked;

  isClicked = button[i].clicked;
  button[i].clicked = false;
  return isClicked;
    
}

//-------------------------------------------------------

void annoy() {
   
  long int timeStamp; 
  byte b = 1;
  byte d = 1;
   
  timeStamp = millis();
  while ((millis() - timeStamp) < ANNOY_TIME) {

    checkButtons();
    if (buttonClicked(0) || buttonClicked(1)) break;

    if (!(millis() % 36)) {
      displayByte(b);
      d ?  b <<= 1 : b >>= 1;
      if (!b) { 
            b = d ? 128 : 1;
            analogWrite(SPKR_PIN, d ? 0 : (random(10) + 1) * 25);
            d = 1 - d;
      }  
      delay(1);
    } 
    
  } 
  
  digitalWrite(SPKR_PIN, LOW);
  setTime(0);
  
}

//-------------------------------------------------------

void checkButtons() {
  
  int reading;
  boolean buttonDown[2];
  long int dt;
  
  reading = analogRead(BTN_PIN);
  buttonDown[0] = reading >= 1000;
  buttonDown[1] = (reading < 800) && (reading > 200);

  for (byte i = 0; i < 2; i++) {
     
     if (buttonDown[i]) {
    
       // button is pressed
       switch (button[i].state) {
         
         // Pressed
         case 0 : button[i].state = 1;
                          button[i].msCount = millis();
                          break;
                           
         // Debounced
         case 1: if (millis() - button[i].msCount > 10) {
                           button[i].state = 2;
                           button[i].clicked = true;
                        }       
                        break;
                        
          // Delayed
          case 2: if (millis() - button[i].msCount > 600)  { 
                           button[i].state = 3;                
                           button[i].msCount = millis();
                           button[i].clicked = true;
                         }   
                         break;          
                           
           // Repeated
           case 3: if (millis() - button[i].msCount > 250)  { 
                            button[i].msCount = millis();
                            button[i].clicked = true;
                          }                          
         
       } 
              
     } else {
        // Button is not pressed
        button[i].state = 0;
        button[i].clicked = false;
     } 
       
  }

}

//-------------------------------------------------------

void loop() {

  long int currentTime = millis();
  
  if (t) {
    if ((currentTime - clockCount) >= MINUTE) {
      setTime(--t);
      if (!t) annoy();
    }  
  } else if (currentTime % (SECOND * 10) < 2) {
                  digitalWrite(XREF_PIN, HIGH); 
                  delay(2);
                  digitalWrite(XREF_PIN, LOW); 
                }  
      
  checkButtons();  
  if (buttonClicked(0)) setTime(t + 1);
  if (buttonClicked(1)) setTime(t << 1);
  
}
